"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const cli_1 = require("@yarnpkg/cli");
const core_1 = require("@yarnpkg/core");
const core_2 = require("@yarnpkg/core");
const clipanion_1 = require("clipanion");
const Constraints_1 = require("../../Constraints");
// eslint-disable-next-line arca/no-default-export
class ConstraintsQueryCommand extends cli_1.BaseCommand {
    constructor() {
        super(...arguments);
        this.json = false;
    }
    async execute() {
        const configuration = await core_1.Configuration.find(this.context.cwd, this.context.plugins);
        const { project } = await core_1.Project.find(configuration, this.context.cwd);
        const constraints = await Constraints_1.Constraints.find(project);
        let query = this.query;
        if (!query.endsWith(`.`))
            query = `${query}.`;
        const report = await core_2.StreamReport.start({
            configuration,
            json: this.json,
            stdout: this.context.stdout,
        }, async (report) => {
            for await (const result of constraints.query(query)) {
                const lines = Array.from(Object.entries(result));
                const lineCount = lines.length;
                const maxVariableNameLength = lines.reduce((max, [variableName]) => Math.max(max, variableName.length), 0);
                for (let i = 0; i < lineCount; i++) {
                    const [variableName, value] = lines[i];
                    report.reportInfo(null, `${getLinePrefix(i, lineCount)}${variableName.padEnd(maxVariableNameLength, ` `)} = ${valueToString(value)}`);
                }
                report.reportJson(result);
            }
        });
        return report.exitCode();
    }
}
ConstraintsQueryCommand.usage = clipanion_1.Command.Usage({
    category: `Constraints-related commands`,
    description: `query the constraints fact database`,
    details: `
      This command will output all matches to the given prolog query.

      If the \`--json\` flag is set the output will follow a JSON-stream output also known as NDJSON (https://github.com/ndjson/ndjson-spec).
    `,
    examples: [[
            `List all dependencies throughout the workspace`,
            `yarn constraints query 'workspace_has_dependency(_, DependencyName, _, _).'`,
        ]],
});
tslib_1.__decorate([
    clipanion_1.Command.Boolean(`--json`)
], ConstraintsQueryCommand.prototype, "json", void 0);
tslib_1.__decorate([
    clipanion_1.Command.String()
], ConstraintsQueryCommand.prototype, "query", void 0);
tslib_1.__decorate([
    clipanion_1.Command.Path(`constraints`, `query`)
], ConstraintsQueryCommand.prototype, "execute", null);
exports.default = ConstraintsQueryCommand;
function valueToString(value) {
    if (typeof value !== `string`)
        return `${value}`;
    if (value.match(/^[a-zA-Z][a-zA-Z0-9_]+$/))
        return value;
    return `'${value}'`;
}
function getLinePrefix(index, count) {
    const isFirst = index === 0;
    const isLast = index === count - 1;
    if (isFirst && isLast)
        return ``;
    if (isFirst)
        return `┌ `;
    if (isLast)
        return `└ `;
    return `│ `;
}
