"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const cli_1 = require("@yarnpkg/cli");
const core_1 = require("@yarnpkg/core");
const clipanion_1 = require("clipanion");
const Constraints_1 = require("../../Constraints");
// eslint-disable-next-line arca/no-default-export
class ConstraintsSourceCommand extends cli_1.BaseCommand {
    constructor() {
        super(...arguments);
        this.verbose = false;
    }
    async execute() {
        const configuration = await core_1.Configuration.find(this.context.cwd, this.context.plugins);
        const { project } = await core_1.Project.find(configuration, this.context.cwd);
        const constraints = await Constraints_1.Constraints.find(project);
        this.context.stdout.write(this.verbose ? constraints.fullSource : constraints.source);
    }
}
ConstraintsSourceCommand.usage = clipanion_1.Command.Usage({
    category: `Constraints-related commands`,
    description: `print the source code for the constraints`,
    details: `
      This command will print the Prolog source code used by the constraints engine. Adding the \`-v,--verbose\` flag will print the *full* source code, including the fact database automatically compiled from your workspaces manifests.
    `,
    examples: [[
            `Prints the source code`,
            `yarn constraints source`,
        ], [
            `Print the source code and the fact database`,
            `yarn constraints source -v`,
        ]],
});
tslib_1.__decorate([
    clipanion_1.Command.Boolean(`-v,--verbose`)
], ConstraintsSourceCommand.prototype, "verbose", void 0);
tslib_1.__decorate([
    clipanion_1.Command.Path(`constraints`, `source`)
], ConstraintsSourceCommand.prototype, "execute", null);
exports.default = ConstraintsSourceCommand;
